@@ -14,6 +14,7 @@ import Control.Monad
 import Data.List(elemIndex)
 import Data.Char(toLower)
 import Data.Maybe ( fromMaybe, fromJust, isJust, isNothing )
+import Data.Time      (UTCTime)
 import Prelude hiding (last)
 import qualified Data.ByteString.Char8 as P
 import qualified Data.ByteString.Char8 as B
@@ -28,33 +29,30 @@ import Happstack.Server.Internal.Types
 import Happstack.Server.Internal.Multipart
 import Happstack.Server.Internal.RFC822Headers
 import Happstack.Server.Internal.MessageWrap
--- import Happstack.Server.Internal.NoPush
 import Happstack.Server.SURI(SURI(..),path,query)
 import Happstack.Server.SURI.ParseURI
+import Happstack.Server.Internal.TimeoutIO (TimeoutIO(..))
 import qualified Happstack.Server.Internal.TimeoutManager as TM
-import Happstack.Server.Internal.TimeoutSocket (sGetContents, sPutTickle, sendFileTickle)
-import Network.Socket (Socket)
-import Network.Socket.ByteString (sendAll)
 import Numeric
 import System.Directory (removeFile)
 import System.IO
 import System.IO.Error (isDoesNotExistError)
 
-request :: TM.Handle -> Conf -> Socket -> Host -> (Request -> IO Response) -> IO ()
-request thandle conf sock host handler = rloop thandle conf sock host handler =<< sGetContents thandle sock
+request :: TimeoutIO -> Maybe (LogAccess UTCTime) -> Host -> (Request -> IO Response) -> IO ()
+request timeoutIO mlog host handler =
+    rloop timeoutIO mlog host handler =<< toGetContents timeoutIO
 
 required :: String -> Maybe a -> Either String a
 required err Nothing  = Left err
 required _   (Just a) = Right a
 
-rloop :: TM.Handle
-         -> Conf
-         -> Socket
+rloop :: TimeoutIO
+         -> Maybe (LogAccess UTCTime)
          -> Host
          -> (Request -> IO Response)
          -> L.ByteString
          -> IO ()
-rloop thandle conf sock host handler inputStr
+rloop timeoutIO mlog host handler inputStr
     | L.null inputStr = return ()
     | otherwise
     = join $
@@ -65,29 +63,29 @@ rloop thandle conf sock host handler inputStr
                       let (m,u,v) = requestLine rql
                       headers' <- parseHeaders "host" (L.unpack headerStr)
                       let headers = mkHeaders headers'
-                      let contentLength = fromMaybe 0 $ fmap fst (P.readInt =<< getHeaderUnsafe contentlengthC headers)
+                      let contentLen = fromMaybe 0 $ fmap fst (P.readInt =<< getHeaderUnsafe contentlengthC headers)
                       (body, nextRequest) <- case () of
-                          () | contentLength < 0               -> fail "negative content-length"
+                          () | contentLen < 0               -> fail "negative content-length"
                              | isJust $ getHeaderBS transferEncodingC headers ->
                                  return $ consumeChunks restStr
-                             | otherwise                       -> return (L.splitAt (fromIntegral contentLength) restStr)
+                             | otherwise                       -> return (L.splitAt (fromIntegral contentLen) restStr)
                       let cookies = [ (cookieName c, c) | cl <- fromMaybe [] (fmap getCookies (getHeader "Cookie" headers)), c <- cl ] -- Ugle
-                      return (m, u, cookies, v, headers, body, host, nextRequest)
+                      return (m, u, cookies, v, headers, body, nextRequest)
 
          case parseRequest of
            Left err -> error $ "failed to parse HTTP request: " ++ err
-           Right (m, u, cookies, v, headers, body, host, nextRequest)
+           Right (m, u, cookies, v, headers, body, nextRequest)
                -> return $
                   do bodyRef        <- newMVar (Body body)
                      bodyInputRef   <- newEmptyMVar
-                     let req = Request m (pathEls (path u)) (path u) (query u)
+                     let req = Request (toSecure timeoutIO) m (pathEls (path u)) (path u) (query u)
                                   (queryInput u) bodyInputRef cookies v headers bodyRef host
 
                      let ioseq act = act >>= \x -> x `seq` return x
 
                      res <- ioseq (handler req) `E.catch` \(e::E.SomeException) -> return $ result 500 $ "Server error: " ++ show e
 
-                     case logAccess conf of
+                     case mlog of
                        Nothing -> return ()
                        (Just logger) ->
                            do time <- getApproximateUTCTime
@@ -95,16 +93,16 @@ rloop thandle conf sock host handler inputStr
                                   user         = "-"
                                   requestLn    = unwords [show $ rqMethod req, rqUri req, show $ rqVersion req]
                                   responseCode = rsCode res
-                                  size         = maybe (-1) (read . B.unpack) (getHeader "Content-Length" res) -- -1 indicates unknown size
+                                  size         = maybe (-1) (readDec' . B.unpack) (getHeader "Content-Length" res) -- -1 indicates unknown size
                                   referer      = B.unpack $ fromMaybe (B.pack "") $ getHeader "Referer" req
                                   userAgent    = B.unpack $ fromMaybe (B.pack "") $ getHeader "User-Agent" req
                               logger host' user time requestLn responseCode size referer userAgent
 
                      -- withNoPush sock $ putAugmentedResult thandle sock req res
-                     putAugmentedResult thandle sock req res
+                     putAugmentedResult timeoutIO req res
                      -- clean up tmp files
                      cleanupTempFiles req
-                     when (continueHTTP req res) $ rloop thandle conf sock host handler nextRequest
+                     when (continueHTTP req res) $ rloop timeoutIO mlog host handler nextRequest
 
 -- NOTE: if someone took the inputs and never put them back, then they are responsible for the cleanup
 cleanupTempFiles :: Request -> IO ()
@@ -207,22 +205,22 @@ method r = fj $ lookup r mtable
 staticHeaders :: Headers
 staticHeaders =
     foldr (uncurry setHeaderBS) (mkHeaders [])
-    [ (serverC, happsC) ]
+    [ (serverC, happstackC) ]
 
 -- FIXME: we should not be controlling the response headers in mysterious ways in this low level code
 -- headers should be set by application code and the core http engine should be very lean.
-putAugmentedResult :: TM.Handle -> Socket -> Request -> Response -> IO ()
-putAugmentedResult thandle outp req res = do
+putAugmentedResult :: TimeoutIO -> Request -> Response -> IO ()
+putAugmentedResult timeoutIO req res = do
     case res of
         -- standard bytestring response
         Response {} -> do
-            let chunked = rsfLength (rsFlags res) == TransferEncodingChunked && isHTTP1_1 req
-            sendTop (if chunked then Nothing else (Just (fromIntegral (L.length (rsBody res))))) chunked
+            let isChunked = rsfLength (rsFlags res) == TransferEncodingChunked && isHTTP1_1 req
+            sendTop (if isChunked then Nothing else (Just (fromIntegral (L.length (rsBody res))))) isChunked
             when (rqMethod req /= HEAD)
-                     (let body = if chunked
+                     (let body = if isChunked
                                  then chunk (rsBody res)
                                  else rsBody res
-                      in sPutTickle thandle outp body)
+                      in toPutLazy timeoutIO body)
         -- zero-copy sendfile response
         -- the handle *should* be closed by the garbage collector
 
@@ -231,25 +229,25 @@ putAugmentedResult thandle outp req res = do
                 off = sfOffset res
                 count = sfCount res
             sendTop (Just count) False
-            TM.tickle thandle
-            sendFileTickle thandle outp infp off count
+            TM.tickle (toHandle timeoutIO)
+            toSendFile timeoutIO infp off count
 
     where ph (HeaderPair k vs) = map (\v -> P.concat [k, fsepC, v, crlfC]) vs
-          sendTop cl chunked = do
-              allHeaders <- augmentHeaders req res cl chunked
-              sendAll outp $ B.concat $ concat
+          sendTop cl isChunked = do
+              allHeaders <- augmentHeaders req res cl isChunked
+              toPut timeoutIO $ B.concat $ concat
                  [ (pversion $ rqVersion req)          -- Print HTTP version
                  , [responseMessage $ rsCode res]      -- Print responseCode
                  , concatMap ph (M.elems allHeaders)   -- Print all headers
                  , [crlfC]
                  ]
-              TM.tickle thandle
+              TM.tickle (toHandle timeoutIO)
           chunk :: L.ByteString -> L.ByteString
           chunk Empty        = LC.pack "0\r\n\r\n"
           chunk (Chunk c cs) = Chunk (B.pack $ showHex (B.length c) "\r\n") (Chunk c (Chunk (B.pack "\r\n") (chunk cs)))
 
 augmentHeaders :: Request -> Response -> Maybe Integer -> Bool -> IO Headers
-augmentHeaders req res mcl chunked = do
+augmentHeaders req res mcl isChunked = do
     -- TODO: Hoist static headers to the toplevel.
     raw <- getApproximateTime
     let stdHeaders = staticHeaders `M.union`
@@ -264,7 +262,7 @@ augmentHeaders req res mcl chunked = do
                                             | otherwise -> []
                               TransferEncodingChunked
                                   -- we check 'chunked' because we might not use this mode if the client is http 1.0
-                                  | chunked   -> [(transferEncodingC, HeaderPair transferEncodingC [chunkedC])]
+                                  | isChunked -> [(transferEncodingC, HeaderPair transferEncodingC [chunkedC])]
                                   | otherwise -> []
 
                      )
@@ -307,7 +305,7 @@ http10 = P.pack "HTTP/1.0"
 http11 :: B.ByteString
 http11 = P.pack "HTTP/1.1"
 
--- Constants
+-- * ByteString Constants
 
 connectionC :: B.ByteString
 connectionC      = P.pack "Connection"
@@ -321,8 +319,8 @@ crlfC :: B.ByteString
 crlfC            = P.pack "\r\n"
 fsepC :: B.ByteString
 fsepC            = P.pack ": "
-contentTypeC :: B.ByteString
-contentTypeC     = P.pack "Content-Type"
+-- contentTypeC :: B.ByteString
+-- contentTypeC     = P.pack "Content-Type"
 contentLengthC :: B.ByteString
 contentLengthC   = P.pack "Content-Length"
 contentlengthC :: B.ByteString
@@ -333,10 +331,10 @@ dateCLower :: B.ByteString
 dateCLower       = P.map toLower dateC
 serverC :: B.ByteString
 serverC          = P.pack "Server"
-happsC :: B.ByteString
-happsC           = P.pack $ "Happstack/" ++ DV.showVersion Paths.version
-textHtmlC :: B.ByteString
-textHtmlC        = P.pack "text/html; charset=utf-8"
+happstackC :: B.ByteString
+happstackC           = P.pack $ "Happstack/" ++ DV.showVersion Paths.version
+-- textHtmlC :: B.ByteString
+-- textHtmlC        = P.pack "text/html; charset=utf-8"
 transferEncodingC :: B.ByteString
 transferEncodingC = P.pack "Transfer-Encoding"
 chunkedC :: B.ByteString
@@ -344,7 +342,7 @@ chunkedC = P.pack "chunked"
 
 -- Response code names
 
-responseMessage :: (Num t) => t -> B.ByteString
+responseMessage :: (Num t, Show t, Eq t) => t -> B.ByteString
 responseMessage 100 = P.pack " 100 Continue\r\n"
 responseMessage 101 = P.pack " 101 Switching Protocols\r\n"
 responseMessage 200 = P.pack " 200 OK\r\n"
