@@ -6,15 +6,16 @@ import Happstack.Server.Internal.Handler        (request)
 import Happstack.Server.Internal.Socket         (acceptLite)
 import Happstack.Server.Internal.TimeoutManager (cancel, initialize, register)
 import Happstack.Server.Internal.TimeoutSocket  as TS
+import qualified Control.Concurrent.Thread.Group as TG
 import Control.Exception.Extensible             as E
 import Control.Concurrent                       (forkIO, killThread, myThreadId)
-import Control.Monad                            (forever, when)
+import Control.Monad
 import Network.BSD                              (getProtocolNumber)
 import Network                                  (sClose, Socket)
-import Network.Socket as Socket (SocketOption(KeepAlive), setSocketOption, 
-                                 socket, Family(..), SockAddr, 
-                                 SocketOption(..), SockAddr(..), 
-                                 iNADDR_ANY, maxListenQueue, SocketType(..), 
+import Network.Socket as Socket (SocketOption(KeepAlive), setSocketOption,
+                                 socket, Family(..), SockAddr,
+                                 SocketOption(..), SockAddr(..),
+                                 iNADDR_ANY, maxListenQueue, SocketType(..),
                                  bindSocket)
 import qualified Network.Socket                 as Socket (listen, inet_addr)
 import System.IO.Error                          (isFullError)
@@ -79,16 +80,19 @@ listen' s conf hand = do
 {-
 #ifndef mingw32_HOST_OS
 -}
-  installHandler openEndedPipe Ignore Nothing
+  void $ installHandler openEndedPipe Ignore Nothing
 {-
 #endif
 -}
   let port' = port conf
+      fork = case threadGroup conf of
+               Nothing -> forkIO
+               Just tg -> \m -> fst `liftM` TG.forkIO tg m
   tm <- initialize ((timeout conf) * (10^(6 :: Int)))
   -- http:// loop
   log' NOTICE ("Listening for http:// on port " ++ show port')
   let eh (x::SomeException) = when ((fromException x) /= Just ThreadKilled) $ log' ERROR ("HTTP request failed with: " ++ show x)
-      work (sock, hn, p) = 
+      work (sock, hn, p) =
           do tid <- myThreadId
              thandle <- register tm (killThread tid)
              let timeoutIO = TS.timeoutSocketIO thandle sock
@@ -97,8 +101,9 @@ listen' s conf hand = do
              cancel thandle
              sClose sock
       loop = forever $ do w <- acceptLite s
-                          forkIO $ work w
+                          fork $ work w
       pe e = log' ERROR ("ERROR in http accept thread: " ++ show e)
+      infi :: IO ()
       infi = loop `catchSome` pe >> infi
 
   infi `finally` (sClose s)
@@ -106,8 +111,7 @@ listen' s conf hand = do
 {--
 #ifndef mingw32_HOST_OS
 -}
-  installHandler openEndedPipe Ignore Nothing
-  return ()
+  void $ installHandler openEndedPipe Ignore Nothing
 {-
 #endif
 -}
